# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-10-09 15:21
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example', '0002_auto_20171009_1719'),
    ]

    operations = [
        migrations.AddField(
            model_name='myuser',
            name='crm_contact_id',
            field=models.UUIDField(blank=True, null=True, verbose_name='CRM contact id'),
        ),
    ]
